package com.swagger.rest.controllers

import com.fasterxml.jackson.module.kotlin.jacksonObjectMapper
import com.swagger.rest.models.*
import com.swagger.rest.models.Enum
import com.swagger.rest.repositories.*
import com.swagger.rest.services.*
import jakarta.servlet.http.HttpServletResponse
import org.springframework.http.HttpHeaders
import org.springframework.http.HttpStatus
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import org.springframework.web.multipart.MultipartFile
import java.io.IOException
import java.text.SimpleDateFormat
import java.util.*
import kotlin.Any
import kotlin.Boolean
import kotlin.ByteArray
import kotlin.RuntimeException
import kotlin.String
import kotlin.Throws
import kotlin.Unit
import kotlin.arrayOf
import kotlin.toString


@RestController
@RequestMapping("/api/v1")
class MaintenanceController(
    private val projectRepository: ProjectRepository,
    private val userRepository: UserRepository,
    private val platformRepository: PlatformRepository,
    private val memberRepository: MemberRepository,
    private val bugRepository: BugRepository,
    private val commentRepository: CommentRepository,
    private val userService: UserService,
    private val projectService: ProjectService,
    private val platformService: PlatformService,
    private val memberService: MemberService,
    private val commentService: CommentService,
    private val bugService: BugService
) {

    @GetMapping("/maintenance", produces = ["text/csv;charset=utf8;"])
    @Throws(IOException::class)
    fun exportToCSV(response: HttpServletResponse, @RequestParam tableName: String): ResponseEntity<*> {
//        val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
//        val valid = memberRepository.validAdmin(userId.id.toString())
        val tableList = listOf("project", "platform", "user", "project_member", "bug", "comment")
//        if (valid == 0) {
//            ResponseEntity<Any>(HttpStatus.FORBIDDEN)
//        } else {
        if (!tableList.contains(tableName) || tableName.isEmpty()) {
            return ResponseEntity<Any>(HttpStatus.BAD_REQUEST)
        } else {
            val list = when (tableName) {
                "user" -> userRepository.findAll().map {
                    "${it.id},\"${it.username}\",\"${it.password}\",\"${it.name}\""
                }

                "project" -> projectRepository.findAll().map {
                    "${it.id},\"${it.name}\",\"${it.description}\",${it.owner!!.id}"
                }

                "platform" -> platformRepository.findAll().map {
                    "${it.id},\"${it.name}\",${it.project!!.id}"
                }

                "project_member" -> memberRepository.findAll().map {
                    "${it.id},${it.role},${it.project!!.id},${it.user!!.id}"
                }

                "bug" -> bugRepository.findAll().map {
                    "${it.id},${SimpleDateFormat("dd-MMM-yy HH:mm:ss").format(it.created)},\"${it.description}\",${it.dev_status},\"${it.goodday_url}\",\"${it.image_url}\",${it.level},${it.status},${it.dev!!.id},${it.platform!!.id},${it.qc!!.id}"
                }

                "comment" -> commentRepository.findAll().map {
                    "${it.id},\"${it.content}\",${SimpleDateFormat("dd-MMM-yy HH:mm:ss").format(it.created)},${it.bug!!.id},${it.creator!!.id}"
                }

                else -> listOf()
            }
            val csvHeader = when (tableName) {
                "user" -> listOf("User ID,\"Username\",\"Password\",\"Name\"")
                "project" -> listOf("Project ID,\"Description\",\"Name\",Owner")
                "platform" -> listOf("Platform ID,\"Name\",Project")
                "project_member" -> listOf("Member ID,Role,Project,User")
                "comment" -> listOf("Bug ID,\"Content\",Created,Bug,Creator")
                "bug" -> listOf(
                    "Bug ID,Created,\"Description\",Dev Status,\"Goodday Url\",\"Image Url\",Level,Status,Dev,Platform,Qc"
                )

                else -> listOf()
            }

            val csvContent = StringBuilder()
            csvContent.append("${csvHeader.joinToString("")}\n")
            csvContent.append(list.joinToString("\n"))
            val headers = HttpHeaders()
//            headers.contentType = MediaType.APPLICATION_OCTET_STREAM
//            headers.setContentDispositionFormData("attachment", "$tableName.csv")
//            val fileName  = String(tableName.toByteArray(), charset("ISO8859-1"))
//            headers["content-disposition"] = "attachment;filename=$fileName.csv"
//            response.addHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=$tableName.csv")
            response.contentType = MediaType.APPLICATION_OCTET_STREAM.toString()
//            response.addHeade
            response.addHeader(
                HttpHeaders.CONTENT_DISPOSITION,
                "attachment; filename=$tableName.csv"
            )
            response.outputStream.write(String(csvContent).toByteArray())
            response.outputStream.flush()
            response.outputStream.close()

            return ResponseEntity("ok", HttpStatus.OK)
//            return ResponseEntity(csvContent, headers, HttpStatus.OK)

//            val cacheResultHandler = CacheDataManager()
//            val inputStream: InputStream
//            val byteArray: ByteArray
//            val httpHeaders = HttpHeaders()
//            val inputStreamResource: InputStreamResource = cacheResultHandler.exportCacheResults(response)
//            httpHeaders[HttpHeaders.CONTENT_DISPOSITION] = "attachment; filename=$tableName.csv"
            //convert inputStream to bytes
//            inputStream = inputStreamResource.inputStream
//            val buffer = ByteArrayOutputStream()
//            var nRead: Int
//            val data = ByteArray(1024)
//            while ((inputStream.read(data, 0, data.size).also { nRead = it }) != -1) {
//                buffer.write(data, 0, nRead)
//            }
//            buffer.flush()
//            byteArray = buffer.toByteArray()
//            httpHeaders.contentLength = byteArray.size.toLong()
//            return ResponseEntity(byteArray, httpHeaders, HttpStatus.OK)
        }
//        return ResponseEntity<Any>(HttpStatus.INTERNAL_SERVER_ERROR)
    }

    @ResponseStatus(HttpStatus.BAD_REQUEST)
    internal class Error : RuntimeException("oxes")

    @PostMapping("/maintenance")
    fun import(
        @RequestParam file: MultipartFile, @RequestParam tableName: String, @RequestParam overwrite: Boolean
    ): ResponseEntity<Any> {
//        val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
//        val valid = memberRepository.validAdmin(userId.id.toString())
        val tableList = listOf("project", "platform", "user", "project_member", "bug", "comment")
//        if (valid == 0) {
//            ResponseEntity<Any>(HttpStatus.FORBIDDEN)
//        } else {
        var found = 0
        val mapData: MutableList<MutableMap<String, Any>> = mutableListOf()
        if (!tableList.contains(tableName) || tableName.isEmpty()) {
            ResponseEntity<Any>(HttpStatus.BAD_REQUEST)
        } else {
            val nameMapping = when (tableName) {
                "user" -> arrayOf("id", "username", "password", "name")
                "project" -> arrayOf("id", "description", "name", "owner")
                "platform" -> arrayOf("id", "name", "project")
                "project_member" -> arrayOf("id", "role", "project", "user")
                "comment" -> arrayOf("id", "content", "created", "bug", "creator")
                "bug" -> arrayOf(
                    "id",
                    "created",
                    "description",
                    "dev_status",
                    "goodday_url",
                    "image_url",
                    "level",
                    "status",
                    "dev",
                    "platform",
                    "qc"
                )

                else -> arrayOf()
            }
            val content = String(file.bytes)
            val lines = content.lines()
            lines.forEachIndexed { ind, t ->
                if (ind == 0 && arrayOf(t) !== nameMapping) {
                    ResponseEntity<Any>(HttpStatus.BAD_REQUEST)
                } else if (t !== "" && ind > 0) {
                    val commas = t.split(",(?=(?:[^\"]*\"[^\"]*\")*[^\"]*$)".toRegex()).toTypedArray()
                    val map: MutableMap<String, Any> = mutableMapOf()
                    nameMapping.forEachIndexed { index, s ->
                        map[s] = commas[index]
                    }
                    mapData.add(map)
                }
            }
            val existing = when (tableName) {
                "project" -> projectRepository.getId()
                "platform" -> platformRepository.getId()
                "user" -> userRepository.getId()
                "project_member" -> memberRepository.getId()
                "bug" -> bugRepository.getId()
                "comment" -> commentRepository.getId()
                else -> listOf()
            }
            mapData.removeIf { p -> existing.contains(p["id"].toString().toLong()) && !overwrite }
            val check = when (tableName) {
                "project" -> mapData.forEach {
                    val projectIn = ProjectInput()
                    projectIn.name = it["name"].toString()
                    projectIn.owner = it["owner"].toString().toLong()
                    projectIn.ownerUsername =
                        userRepository.findById(projectIn.owner!!.toLong()).orElseThrow { Error() }.username
                    projectIn.id = it["id"].toString().toLong()
                    projectIn.description = it["description"].toString()
                    val validate = projectService.validate(projectIn)
                    found += if (validate !== HttpStatus.OK && validate !== HttpStatus.CONFLICT) 1 else 0
                }

                "platform" -> mapData.forEach {
                    val platformInput = PlatformInput()
                    platformInput.id = it["id"].toString().toLong()
                    platformInput.name = it["name"].toString()
                    platformInput.project_id = it["project"].toString().toLong()
                    platformInput.project_name =
                        projectRepository.findById(platformInput.project_id!!.toLong()).orElseThrow { Error() }.name
                    val validate = platformService.validate(platformInput)
                    found += if (validate !== HttpStatus.OK && validate !== HttpStatus.CONFLICT && validate !== HttpStatus.FORBIDDEN) 1 else 0
                }

                "user" -> mapData.forEach {
                    val validate = userService.validate(
                        jacksonObjectMapper().readValue(
                            jacksonObjectMapper().writeValueAsString(it), UserInput::class.java
                        )
                    )
                    found += if (validate !== HttpStatus.OK && validate !== HttpStatus.CONFLICT) 1 else 0
                }

                "project_member" -> mapData.forEachIndexed { _, m ->
                    val memberInput = MemberInput()
                    memberInput.id = m["id"].toString().toLong()
                    memberInput.roleOrdinal = Integer.parseInt(m["role"].toString())
                    memberInput.role = if (Enum.Member.values()
                            .any { it.ordinal == memberInput.roleOrdinal }
                    ) Enum.Member.values()[memberInput.roleOrdinal!!].name else throw Error()
                    memberInput.user_id = m["user"].toString().toLong()
                    val validate = memberService.validate(null, memberInput)
                    found += if (validate !== HttpStatus.OK && validate !== HttpStatus.CONFLICT && validate !== HttpStatus.FORBIDDEN) 1 else 0
                }

                "comment" -> mapData.forEach {
                    val commentInput = CommentInput()
                    commentInput.id = it["id"].toString().toLong()
                    commentInput.bug_id = it["bug"].toString().toLong()
                    commentInput.bug = bugRepository.findById(commentInput.bug_id!!).orElseThrow { Error() }.description
                    commentInput.content = it["content"].toString()
                    commentInput.creator = it["creator"].toString()
                    commentInput.created = it["created"].toString()
                    val validate = commentService.validate(commentInput.bug_id, commentInput)
                    found += if (validate !== HttpStatus.OK) 1 else 0
                }

                "bug" -> mapData.forEachIndexed { _, m ->
                    val bugInput = BugInput()
                    bugInput.id = m["id"].toString().toLong()
                    bugInput.created = m["created"].toString()
                    bugInput.description = m["description"].toString()
                    bugInput.devStatOrdinal = Integer.parseInt(m["dev_status"].toString())
                    bugInput.dev_status = if (Enum.Dev_Status.values()
                            .any { it.ordinal == bugInput.devStatOrdinal }
                    ) Enum.Dev_Status.values()[bugInput.devStatOrdinal!!].name else throw Error()
                    bugInput.levelOrdinal = Integer.parseInt(m["level"].toString())
                    bugInput.level = if (Enum.Level.values()
                            .any { it.ordinal == bugInput.levelOrdinal }
                    ) Enum.Level.values()[bugInput.levelOrdinal!!].name else throw Error()
                    bugInput.statusOrdinal = Integer.parseInt(m["status"].toString())
                    bugInput.status = if (Enum.Status.values()
                            .any { it.ordinal == bugInput.statusOrdinal }
                    ) Enum.Status.values()[bugInput.statusOrdinal!!].name else throw Error()
                    bugInput.goodday_url = m["goodday_url"].toString()
                    bugInput.image_url = m["image_url"].toString().drop(1).dropLast(1)
                    bugInput.platformId = m["platform"].toString().toLong()
                    bugInput.platform = platformRepository.findById(bugInput.platformId).orElseThrow { Error() }.name
                    bugInput.dev = m["dev"].toString().toLong()
                    bugInput.qc = m["qc"].toString().toLong()
                    val validate = bugService.validate(bugInput)
                    found += if (validate !== HttpStatus.OK && validate !== HttpStatus.CONFLICT && validate !== HttpStatus.FORBIDDEN) 1 else 0
                }

                else -> Unit
            }
        }
        val project = Project()
        val platform = Platform()
        val user = User()
        val member = ProjectMember()
        val comment = Comment()
        val bug = Bug()
        if (found == 0) {
            val unit = when (tableName) {
                "project" -> mapData.forEach {
                    project.id = it["id"].toString().toLong()
                    project.name = it["name"].toString().dropLast(1).drop(1)
                    project.description = it["description"].toString().dropLast(1).drop(1)
                    project.owner = userRepository.findById(it["owner"].toString().toLong()).get()
                    projectRepository.save(project)
                }

                "user" -> mapData.forEach {
                    user.id = it["id"].toString().toLong()
                    user.name = it["name"].toString().dropLast(1).drop(1)
                    user.password = it["password"].toString().dropLast(1).drop(1)
                    user.username = it["username"].toString().dropLast(1).drop(1)
                    userRepository.save(user)
                }

                "platform" -> mapData.forEach {
                    platform.id = it["id"].toString().toLong()
                    platform.name = it["name"].toString().dropLast(1).drop(1)
                    platform.project = projectRepository.findById(it["project"].toString().toLong()).get()
                    platformRepository.save(platform)
                }

                "project_member" -> mapData.forEach {
                    member.id = it["id"].toString().toLong()
                    member.role = Enum.Member.values()[Integer.parseInt(it["role"].toString())].ordinal
                    member.project = projectRepository.findById(it["project"].toString().toLong()).get()
                    member.user = userRepository.findById(it["user"].toString().toLong()).get()
                    memberRepository.save(member)
                }

                "comment" -> mapData.forEach {
                    comment.id = it["id"].toString().toLong()
                    comment.content = it["content"].toString().dropLast(1).drop(1)
                    comment.created =
                        SimpleDateFormat("dd-MMM-yy HH:mm:ss", Locale.ENGLISH).parse(it["created"].toString())
                    comment.bug = bugRepository.findById(it["bug"].toString().toLong()).get()
                    comment.creator = userRepository.findById(it["creator"].toString().toLong()).get()
                    commentRepository.save(comment)
                }

                "bug" -> mapData.forEach {
                    bug.id = it["id"].toString().toLong()
                    bug.created = SimpleDateFormat("dd-MMM-yy HH:mm:ss", Locale.ENGLISH).parse(it["created"].toString())
                    bug.description = it["description"].toString().dropLast(1).drop(1)
                    bug.dev_status = Enum.Dev_Status.values()[Integer.parseInt(it["dev_status"].toString())].ordinal
                    bug.goodday_url = it["goodday_url"].toString().dropLast(1).drop(1)
                    bug.image_url = it["image_url"].toString().dropLast(1).drop(1)
                    bug.level = Enum.Level.values()[Integer.parseInt(it["level"].toString())].ordinal
                    bug.status = Enum.Status.values()[Integer.parseInt(it["status"].toString())].ordinal
                    bug.dev = userRepository.findById(it["dev"].toString().toLong()).get()
                    bug.platform = platformRepository.findById(it["platform"].toString().toLong()).get()
                    bug.qc = userRepository.findById(it["qc"].toString().toLong()).get()
                    bugRepository.save(bug)
                }

                else -> Unit
            }
        }
        return if (found == 0) ResponseEntity<Any>(HttpStatus.OK) else ResponseEntity<Any>(HttpStatus.BAD_REQUEST)
    }

}


