package com.swagger.rest.services

import com.swagger.rest.models.Enum
import com.swagger.rest.models.MemberInput
import com.swagger.rest.models.Project
import com.swagger.rest.models.ProjectMember
import com.swagger.rest.repositories.MemberRepository
import com.swagger.rest.repositories.ProjectRepository
import com.swagger.rest.repositories.UserRepository
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpStatus
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.stereotype.Service
import java.util.*

@Service
class MemberService {

    @Autowired
    private val projectRepository: ProjectRepository? = null

    @Autowired
    private val userRepository: UserRepository? = null

    @Autowired
    private val memberRepository: MemberRepository? = null

    fun saveMember(id: Long? = null, memberInput: MemberInput): ProjectMember? {
        return try {
            val save = ProjectMember()
            save.project = projectRepository!!.findById(id!!).get()
            save.user = userRepository!!.findById(memberInput.user_id!!.toLong()).get()
            save.role = Enum.Member.valueOf(memberInput.role!!.uppercase()).ordinal
            memberRepository!!.save(save)
        } catch (e: Exception) {
            e.printStackTrace()
            null
        }
    }

    fun validate(id: Long? = null, memberInput: MemberInput): HttpStatus? {
        try {
            val userId = userRepository!!.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
            var project: Optional<Project>? = null
            var validOwner = 0
            var validAdmin = 0
            if (id !== null) {
                project = projectRepository!!.findById(id)
            }
            val user = userRepository.findById(memberInput.user_id!!.toLong())
            return if (user.isPresent || (id !== null && project!!.isPresent)) {
                if (id !== null) {
                    validOwner = projectRepository!!.validOwner(project!!.get().id.toString(), userId.id.toString())
                    validAdmin = memberRepository!!.validRole(project.get().id.toString(), userId.id.toString(), "2")
                }
                val role = Enum.Member.valueOf(memberInput.role!!.uppercase()).ordinal
                val found =
                    memberRepository!!.findDup(id.toString(), role.toString(), memberInput.user_id.toString()).size
                if (!enumValues<Enum.Member>().any { it.name == memberInput.role!!.uppercase() }) {
                    HttpStatus.BAD_REQUEST
                } else if (!(validOwner > 0 || validAdmin > 0)) {
                    HttpStatus.FORBIDDEN
                } else if (found > 0) {
                    HttpStatus.CONFLICT
                } else {
                    HttpStatus.OK
                }
            } else {
                HttpStatus.NOT_FOUND
            }
        } catch (e: Exception) {
            e.printStackTrace()
            return HttpStatus.INTERNAL_SERVER_ERROR
        }
    }

}