package com.swagger.rest.services

import com.swagger.rest.models.Project
import com.swagger.rest.models.ProjectInput
import com.swagger.rest.repositories.ProjectRepository
import com.swagger.rest.repositories.UserRepository
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpStatus
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.stereotype.Service

@Service
class ProjectService {

    @Autowired
    private val projectRepository: ProjectRepository? = null

    @Autowired
    private val userRepository: UserRepository? = null

    fun saveProject(projectInput: ProjectInput): Project? {
        return try {
            val save = Project()
            save.name = projectInput.name.trim()
            save.description = projectInput.description!!.trim()
            save.owner = userRepository!!.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
            projectRepository!!.save(save)
        } catch (e: Exception) {
            e.printStackTrace()
            null
        }
    }

    fun updateProject(id: Long, projectInput: ProjectInput): Project? {
        return try {
            val save = projectRepository!!.findById(id).get()
            save.name = projectInput.name.trim()
            save.description = projectInput.description!!.trim()
            projectRepository.save(save)
        } catch (e: Exception) {
            e.printStackTrace()
            null
        }
    }

    fun validate(projectInput: ProjectInput): HttpStatus? {
        try {
            val found = projectRepository!!.countByName(projectInput.name)
            val userId = userRepository!!.countByUser(projectInput.owner.toString(), projectInput.ownerUsername)
            return if (projectInput.name.isBlank() || (userId == 0 && projectInput.ownerUsername.isNotEmpty())) {
                HttpStatus.BAD_REQUEST
            } else if (projectInput.name.length > 100 || projectInput.description!!.length > 255) {
                HttpStatus.PAYLOAD_TOO_LARGE
            } else if (found > 0) {
                HttpStatus.CONFLICT
            } else {
                HttpStatus.OK
            }
        } catch (e: Exception) {
            e.printStackTrace()
            return HttpStatus.INTERNAL_SERVER_ERROR
        }
    }

}