package com.swagger.rest.services

import com.swagger.rest.models.User
import com.swagger.rest.models.UserInput
import com.swagger.rest.repositories.UserRepository
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpStatus
import org.springframework.security.crypto.password.PasswordEncoder
import org.springframework.stereotype.Service

@Service
class UserService {

    @Autowired
    private val passwordEncoder: PasswordEncoder? = null

    @Autowired
    private val userRepository: UserRepository? = null

    fun saveUser(userInput: UserInput): User? {
        return try {
            val save = User()
            save.username = userInput.username!!.trim()
            save.name = userInput.name!!
            save.password = passwordEncoder!!.encode(userInput.password)
            userRepository!!.save(save)
        } catch (e: Exception) {
            e.printStackTrace()
            null
        }
    }

    fun updateUser(id: Long, userInput: UserInput, update: Boolean, pass: Boolean): User? {
        return try {
            val save = userRepository!!.findById(id).get()
            if (!pass) {
                save.username = userInput.username!!.trim()
                save.name = userInput.name!!
            }
            if (!update) {
                save.password = passwordEncoder!!.encode(userInput.newPassword)
            }
            userRepository.save(save)
        } catch (e: Exception) {
            e.printStackTrace()
            null
        }
    }

    fun validate(userInput: UserInput): HttpStatus? {
        return try {
            val found = userRepository!!.findByUsername(userInput.username!!).size
            if (userInput.username!!.isBlank()) {
                HttpStatus.BAD_REQUEST
            } else if (userInput.username!!.length > 100 || userInput.password!!.length > 100 || userInput.name!!.length > 255) {
                HttpStatus.PAYLOAD_TOO_LARGE
            } else if (found > 0) {
                HttpStatus.CONFLICT
            } else {
                HttpStatus.OK
            }
        } catch (e: Exception) {
            e.printStackTrace()
            HttpStatus.INTERNAL_SERVER_ERROR
        }
    }

}